<?php

namespace Litepie\Example\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litepie\Example\Http\Requests\PermissionRequest;
use Litepie\Example\Interfaces\PermissionRepositoryInterface;
use Litepie\Example\Models\Permission;

/**
 * Resource controller class for permission.
 */
class PermissionResourceController extends BaseController
{

    /**
     * Initialize permission resource controller.
     *
     * @param type PermissionRepositoryInterface $permission
     *
     * @return null
     */
    public function __construct(PermissionRepositoryInterface $permission)
    {
        parent::__construct();
        $this->repository = $permission;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litepie\Example\Repositories\Criteria\PermissionResourceCriteria::class);
    }

    /**
     * Display a list of permission.
     *
     * @return Response
     */
    public function index(PermissionRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litepie\Example\Repositories\Presenter\PermissionPresenter::class)
                ->$function();
        }

        $permissions = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('example::permission.names'))
            ->view('example::permission.index', true)
            ->data(compact('permissions', 'view'))
            ->output();
    }

    /**
     * Display permission.
     *
     * @param Request $request
     * @param Model   $permission
     *
     * @return Response
     */
    public function show(PermissionRequest $request, Permission $permission)
    {

        if ($permission->exists) {
            $view = 'example::permission.show';
        } else {
            $view = 'example::permission.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('example::permission.name'))
            ->data(compact('permission'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new permission.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PermissionRequest $request)
    {

        $permission = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('example::permission.name')) 
            ->view('example::permission.create', true) 
            ->data(compact('permission'))
            ->output();
    }

    /**
     * Create new permission.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PermissionRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $permission                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('example::permission.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('example/permission/' . $permission->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/example/permission'))
                ->redirect();
        }

    }

    /**
     * Show permission for editing.
     *
     * @param Request $request
     * @param Model   $permission
     *
     * @return Response
     */
    public function edit(PermissionRequest $request, Permission $permission)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('example::permission.name'))
            ->view('example::permission.edit', true)
            ->data(compact('permission'))
            ->output();
    }

    /**
     * Update the permission.
     *
     * @param Request $request
     * @param Model   $permission
     *
     * @return Response
     */
    public function update(PermissionRequest $request, Permission $permission)
    {
        try {
            $attributes = $request->all();

            $permission->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('example::permission.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('example/permission/' . $permission->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('example/permission/' . $permission->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the permission.
     *
     * @param Model   $permission
     *
     * @return Response
     */
    public function destroy(PermissionRequest $request, Permission $permission)
    {
        try {

            $permission->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('example::permission.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('example/permission/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('example/permission/' . $permission->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple permission.
     *
     * @param Model   $permission
     *
     * @return Response
     */
    public function delete(PermissionRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('example::permission.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('example/permission'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/example/permission'))
                ->redirect();
        }

    }

    /**
     * Restore deleted permissions.
     *
     * @param Model   $permission
     *
     * @return Response
     */
    public function restore(PermissionRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('example::permission.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/example/permission'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/example/permission/'))
                ->redirect();
        }

    }

}
